@@ -136,16 +136,15 @@ impl OpenVpn {
 
             pos += x;
 
-            if let Some(cap) = dns_regex.captures(&buffer) {
-                if openvpn_dns.is_none() {
-                    if let Some(ipstr) = cap.get(1) {
-                        debug!("Found OpenVPN DNS response: {}", ipstr.as_str());
-                        let ipaddr = IpAddr::from_str(ipstr.as_str());
-                        if let Ok(ip) = ipaddr {
-                            openvpn_dns = Some(ip);
-                            debug!("Set OpenVPN DNS to: {ip:?}");
-                        }
-                    }
+            if let Some(cap) = dns_regex.captures(&buffer)
+                && openvpn_dns.is_none()
+                && let Some(ipstr) = cap.get(1)
+            {
+                debug!("Found OpenVPN DNS response: {}", ipstr.as_str());
+                let ipaddr = IpAddr::from_str(ipstr.as_str());
+                if let Ok(ip) = ipaddr {
+                    openvpn_dns = Some(ip);
+                    debug!("Set OpenVPN DNS to: {ip:?}");
                 }
             }
 
