@@ -113,10 +113,7 @@ impl Wireguard {
                 "ip6"
             };
 
-            debug!(
-                "Opening firewall for Wireguard peer (out): {} dport {}",
-                peer_ip_str, peer_port
-            );
+            debug!("Opening firewall for Wireguard peer (out): {peer_ip_str} dport {peer_port}");
             // Allow the initial OUTGOING connection packet.
             NetworkNamespace::exec(
                 &namespace.name,
@@ -138,10 +135,7 @@ impl Wireguard {
                 ],
             )?;
 
-            debug!(
-                "Opening firewall for Wireguard peer (in): {} sport {}",
-                peer_ip_str, peer_port
-            );
+            debug!("Opening firewall for Wireguard peer (in): {peer_ip_str} sport {peer_port}");
             // Allow the server's INCOMING reply packet.
             NetworkNamespace::exec(
                 &namespace.name,
